#!/usr/bin/env node

const program = require('commander');
const { get } = require('lodash');

const cwd = process.cwd();
const packageJson = require('../package.json');
const version = get(packageJson, 'version');

program
  .version(version, '-v, --version');

program
  .command('sync')
  .action(() => {
    require('../lib/sync')(cwd);
  });

program
  .command('diff')
  .argument('[module-key]', 'module key such as "function:sum"')
  .option('-d, --details', 'difference details')
  .action((moduleKey, option) => {
    require('../lib/diff')(cwd, moduleKey, option.details);
  });

program
  .command('copy')
  .argument('<module-key>', 'module key such as "function:sum"')
  .argument('<target>', 'target path')
  .action((moduleKey, target, option) => {
    require('../lib/copy')(cwd, moduleKey, target);
  });

program.parse(process.argv);
