const path = require('path');
const fs = require('fs');
const { diffLines } = require('diff');
const getModuleEntry = require('../utils/getModuleEntry');
const getModule = require('../utils/getModule');
const recursionDir = require('../utils/recursionDir');
const md5File = require('md5-file')

module.exports = (cwd, moduleKey, detailsVisible) => {
  const base = cwd;
  const fileList = recursionDir(base, true, (item) => {
    if (item.match(/\.DS_Store$/)) {
      return false;
    }
    if (item.match(/node_modules/)) {
      return false;
    }
    return true;
  });

  let moduleList = [];
  fileList.forEach(item => {
    const module = getModule(path.resolve(base, item));
    if (module) {
      moduleList.push(module);
    }
  });

  console.log('[workspace]', cwd);

  // 指定模块，过滤
  if (moduleKey) {
    moduleList = moduleList.filter(o => o.module === moduleKey);
    if (moduleList.length === 0) {
      console.log(`can\'t find the module "${moduleKey}"`);
      return;
    } else {
      detailsVisible = true;
    }
  }

  // 过滤源目录
  moduleList = moduleList.filter(targetModule => getModule(getModuleEntry(targetModule.module)).base !== targetModule.base);
  moduleList.sort((a, b) => a.module < b.module ? -1 : 1);

  moduleList.forEach((targetModule, index) => {
    const sourceModuleEntry = getModuleEntry(targetModule.module);
    const sourceModule = getModule(sourceModuleEntry);

    console.log(`[${index + 1}/${moduleList.length}]`, targetModule.module,
      `"${path.relative(cwd, targetModule.base)}"`,
      `(${targetModule.files.length}/${sourceModule.files.length})`);
    sourceModule.files.forEach(sourceFile => {
      let targetFile;
      if (sourceModule.type === 0) {
        targetFile = targetModule.entry;
      } else {
        targetFile = targetModule.files.filter(o => o === sourceFile)[0];
      }

      if (!targetFile) {
        console.log('ADD', sourceFile);
        return;
      }

      const sourcePath = path.resolve(sourceModule.base, sourceFile);
      const targetPath = path.resolve(targetModule.base, targetFile);
      const md5File = require('md5-file');
      if (md5File.sync(sourcePath) !== md5File.sync(targetPath)) {
        console.log('MOD', sourceFile);
      }

      if (!detailsVisible) {
        return;
      }

      // 大文件不进行详情diff
      if (fs.statSync(sourcePath).size > 1024 * 500) {
        return;
      }

      const sourceStr = fs.readFileSync(sourcePath, 'utf8');
      const targetStr = fs.readFileSync(targetPath, 'utf8');
      const result = diffLines(targetStr, sourceStr);
      const diff = result.filter(o => o.added || o.removed);
      diff.forEach((part, i) => {
        if (part.added) {
          part.value.split('\n').forEach(item => {
            console.log(`+${item}`);
          });
        }
        if (part.removed) {
          part.value.split('\n').forEach(item => {
            console.log(`-${item}`);
          });
        }
      });
    });
  });
};
